<?php

/***************************************************************************
 *   Copyright (C) 2004 by Thomas Flueeli                                  *
 *   cairol@thatrix.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/ 
 
/***************************************************************************
 * Version                                                                 *
 *   $Id: phpGuestbook.php,v 1.16 2004/05/18 08:01:54 thatrix Exp $                *
 ***************************************************************************/ 

/***************************************************************************
 * History                                                                 *
 *   06.04.2003 : Initial release [thatrix]                                *
 *   07.04.2003 : PEAR:DB Support [thatrix]                                *
 *   10.04.2003 : Template Engine [thatrix]                                *
 *   08.05.2003 : Smilies [thatrix]                                        *
 *   13.05.2003 : Text Wrapping [thatrix]                                  *
 *   14.05.2003 : Global Config [thatrix]                                  *
 *   15.05.2003 : Improved Smilie Parser [thatrix]                         *
 *   19.05.2003 : Merged checkPost and checkGet together [thatrix]         *
 *   26.05.2003 : Absolute pathnames [thatrix]                             *
 *   28.05.2003 : Removed wrapText [thatrix]                               *
 *   31.07.2003 : "Pages Link Bar" generation (calcPages) [thatrix]        *
 *   17.11.2003 : fetchPosts [thatrix]                                     *
 *   18.11.2003 : Multi language support [thatrix]                         *
 *   20.11.2003 : Install Code [thatrix]                                   *
 *   24.11.2003 : Install and loadConfig bugfixes [thatrix]                *
 *   25.11.2003 : Table prefix [thatrix]                                   *
 *   06.01.2004 : Bugfixes (PLB) [thatrix]                                 *
 *   16.02.2004 : Added formats [thatrix]                                  *
 *   18.02.2004 : Performance tuning [thatrix]                             *
 *   18.05.2004 : parseSmiliesInvert() [thatrix]                           *
 ***************************************************************************/ 

 if (!defined('IN_PHPGB')) {
        die('Hacking attempt');
 }

 // Load config file
 include($phpGB_base . 'config.php'); 

 // Load Libraries
 require_once($config['Smarty']);       // Smarty
 require_once($config['PEAR_DB']);      // PEAR::DB

 // Error Handling
 PEAR::setErrorHandling(PEAR_ERROR_TRIGGER);                                    

 if (defined('PHPGB_INSTALLED')) {
    // Connect to Datbase
    $DB = DB::connect($config['DSN'], false);

    // Load Config
    loadConfig();
 
    // Import language
    $lang = array();
    require_once($phpGB_base . 'language/' . $config['Language'] . '/lang.php');
 }
 
 // Smarty Settings
 $template = new Smarty();
 $template->template_dir  = $phpGB_base . 'templates';
 $template->compile_dir   = $phpGB_base . 'compile';
 $template->config_dir    = $phpGB_base . 'configs';
 $template->caching       = false;
 $template->compile_check = false;
 $template->force_compile = false;
 $template->security      = true;
 $template->secure_dir    = array();
 $template->use_sub_dirs  = $config['Smarty_use_sub_dirs'];
 
 // Smilie Parser
 $SP_searches = NULL;
 $SP_replaces = NULL;

 /********************************************************************
  * Function    : writeHeader
  * Description : Writes the HTTP header and disables Caching.
  * Params      : -
  * Return      : -
  ********************************************************************/
 function writeHeader()
 {
    global $lang;
    
        // HTTP Header
        header("Content-language: " . $lang['LANGUAGE']);
        header("Content-type: text/html;");
            
        // No Caching
        header("Expires: Mon, 10 Jul 1978 05:00:00 GMT"             );
        header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
        header("Cache-Control: no-store, no-cache, must-revalidate" );  // HTTP 1.1
        header("Cache-Control: post-check=0, pre-check=0", FALSE    ); 
        header("Pragma: no-cache"                                   );  // HTTP 1.0
 }

 /********************************************************************
  * Function    : loadConfig
  * Description : Loads configuration settings from database.
  * Params      : -
  * Return      : -
  ********************************************************************/
 function loadConfig()
 {
     global $DB, $config;
     
     $result = $DB->query('SELECT * FROM ' . $config['prefix'] . 'Config');
     while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
        $config[$row['ConfigName']] = $row['ConfigValue'];
     }
     $result->free();
 }
 
 /********************************************************************
  * Function    : fetchPosts
  * Description : Fetches all posts.
  * Params      : -
  * Return      : -
  ********************************************************************/
 function fetchPosts()
 {
     global $DB, $config, $pages, $lang;
     $result = $DB->query('SELECT PostID, IP, Name, EMail, Website, Post, date_format(Date, \'' . $lang['DATE_FORMAT'] . '\') AS FDate FROM ' . $config['prefix'] . 'Posts ORDER BY Date DESC LIMIT ' . $pages[2] . ', ' . $config['PostsPerPage']);
     $allPosts = array();
     while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
        $allPosts[] = array('Mail'     => $row['EMail'],
                                        'Website'  => $row['Website'],
                                            'Date'     => $row['FDate'],
                                            'Time'     => $row['Time'],
                                            'Name'     => $row['Name'],
                                            'Post'     => $row['Post'],
                                            'PostID'   => $row['PostID'],
                                            'IP'       => $row['IP']);
     }
     $result->free();
     
     return $allPosts;
 }

 /********************************************************************
  * Function    : checkPG
  * Description : Checks if a POST/GET Variable exist isn't empty and 
  *                       optionally if it's nummeric.
  * Params      : Mode, Post Name, [check if isNummeric]
  * Return      : True or False
  ********************************************************************/
 function checkPG($varName = '', $mode = 0, $isNummeric = false)
 {
        $typeOk = true;
        
        if ($mode === 0) {
                $PG = $_POST;
        } else {
                $PG = $_GET;
        }
                
        if (isset($PG[$varName]) && trim($PG[$varName]) != '') {
                if ($isNummeric === true) {
                        $typeOk = is_numeric($PG[$varName]);
                }
                return $typeOk;
        }
                
        return false;
 }

 /********************************************************************
  * Function    : getSmilies
  * Description : Reads all Smilies from the Database and return an
  *                       associative array with filename, code and desc.
  * Params      : [limit]
  * Return      : NULL or Array
  ********************************************************************/
 function getSmilies($limit = -1)
 {
        global $DB, $config;
        
        if (!is_int($limit)) {
                return null;
        }

        $result = $DB->query('SELECT Code, Description, Filename FROM ' . $config['prefix'] . 'Smilies GROUP BY Filename ORDER BY SmiliesId LIMIT ' . $limit);
        
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
                $allSmilies[] = array('Filename'    => $config['SmiliesPath'] . $row['Filename'],
                                                          'Code'        => $row['Code'],
                                                          'Description' => $row['Description']);
        }
        $result->free();

        return $allSmilies;
 }
 
 /********************************************************************
  * Function    : startSmilieParser
  * Description : Get all Smilies from Database and sort the arrays.
  * Params      : -
  * Return      : -
  ********************************************************************/
 function startSmilieParser(){
        global $DB, $config, $SP_searches, $SP_replaces;
        
        // get smilies
        $result = $DB->query('SELECT Code, Description, Filename FROM ' . $config['prefix'] . 'Smilies ORDER BY LENGTH(Code) DESC');
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
         $SP_searches[] = $row['Code'];
         $SP_replaces[] = '<img src="'. $config['URL'] . $config['SmiliesPath'] . $row['Filename'] . '" alt="' . $row['Description'] . '" border="0">';
        }
        $result->free();
        
        // Format
        $SP_searches[] = '[b]';         // Bold
        $SP_replaces[] = '<b>';
        $SP_searches[] = '[/b]';
        $SP_replaces[] = '</b>';
        $SP_searches[] = '[i]';         // Italic
        $SP_replaces[] = '<i>';
        $SP_searches[] = '[/i]';
        $SP_replaces[] = '</i>';
        $SP_searches[] = '[u]';         // Underline
        $SP_replaces[] = '<u>';
        $SP_searches[] = '[/u]';
        $SP_replaces[] = '</u>';

        ksort($SP_searches);
        ksort($SP_replaces); 
 }

 /********************************************************************
  * Function    : parseSmilies
  * Description : Replaces all Smilie Codes in the text with HTML
  *                       <img> tags.
  * Params      : Text with Smilie Codes
  * Return      : Text with Image Tags
  ********************************************************************/
 function parseSmilies($text)
 {
        global $SP_searches, $SP_replaces;
        
        if (count($SP_searches)) {
                $text = str_replace($SP_searches, $SP_replaces, $text);
        }
        return $text;
 }
 
 /********************************************************************
  * Function    : parseSmiliesInverse
  * Description : Replaces all <img> tags with the Smilie Codes 
  * Params      : Text with Smilie Codes
  * Return      : Text with Image Tags
  ********************************************************************/
 function parseSmiliesInverse($text)
 {
        global $SP_searches, $SP_replaces;
        
        if (count($SP_searches)) {
                $text = str_replace($SP_replaces, $SP_searches, $text);
        }
        return $text;
 }
  
 /********************************************************************
  * Function    : calcPages
  * Description : Calculates the Pages Link Bar.
  * Params      : -
  * Return      : Array that contains:
  *                             [0] => Number of posts
  *                             [1] => Number of pages
  *                             [2] => Current offset
  *                             [3] => Pages Link Bar
  ********************************************************************/
 function calcPages($admin = false)
 {
        global $DB, $config, $lang;
        
        // Pages
    if (checkPG('page', 1, true)) {
                $offset = $config['PostsPerPage'] * $_GET['page'] - $config['PostsPerPage'];
        } else {
                $offset = 0;
                $_GET['page'] = 1;
        }

        // Count Posts
        $result = $DB->query('SELECT COUNT(PostID) AS Anzahl FROM ' . $config['prefix'] . 'Posts');
        $row    = $result->fetchRow(DB_FETCHMODE_ASSOC);
        $count  = $row['Anzahl'];
        $pages  = ceil($count / $config['PostsPerPage']);
        $result->free();

        $pagesLink = '';
        $start     = 1;
        if ($pages <= 4) {
                $end = $pages;
        } else {
                $end = 4;
        }
         
        // If current page is greater than 3 => curr-2, curr-1, CURR, curr+1, curr+2
        if ($_GET['page'] > 3) {
                $start     =  $_GET['page'] - 2;
                $end       =  $_GET['page'] + 2;
                if($admin){ 
                    $pagesLink .= '<a href="' . $config['URL'] . 'admin.php?site=edit&page=1">&lt;&lt; ' . $lang['First'] . '</a> ... ';
                } else {
                    $pagesLink .= '<a href="' . $config['URL'] . 'index.php?page=1">&lt;&lt; ' . $lang['First'] . '</a> ... ';
                }
                if ($end > $pages) {
                        $end   =  $pages;
                    if ($end == $_GET['page']) $start -= 1;
                }
        }
          
        // Generate Pages Link
        for ($i = $start; $i <= $end; $i++) {
                if ($i == $_GET['page']) {
                        $pagesLink .= '<b>[' . $i . ']</b> ';
                } else {
                if($admin){ 
                        $pagesLink .= '<a href="' . $config['URL'] . 'admin.php?site=edit&page=' . $i . '">' . $i . '</a> ';
                } else {
                        $pagesLink .= '<a href="' . $config['URL'] . 'index.php?page=' . $i . '">' . $i . '</a> ';
                }
                }
        }
        
        // Last
        if ($end < $pages) {
                if($admin){ 
                $pagesLink .= '... <a href="' . $config['URL'] . 'admin.php?site=edit&page=' . $pages . '">' . $lang['Last'] . ' &gt;&gt;</a>';
                } else {
                $pagesLink .= '... <a href="' . $config['URL'] . 'index.php?page=' . $pages . '">' . $lang['Last'] . ' &gt;&gt;</a>';
        }
        }

        return array($count, $pages, $offset, $pagesLink);
 }
 
?>